@@ -57,7 +57,7 @@ BOOL nt_init_board_menu()
 	}
 
 
-	if(!nt_http_get(url, outp, NULL, NULL, NULL)){
+	if(!nt_http_get(url, outp, NULL, NULL, NULL, FALSE)){
 		free(outp);
 		return FALSE;
 	}
@@ -85,7 +85,7 @@ BOOL nt_init_board_menu()
 		while(0	<	n_src){
 			size_t sz = iconv(icd, &p_src, &n_src, &p_dst, &n_dst);
 			if(sz == -1){
-				fprintf(stderr, "errno: %d %s\n", errno, s_src);
+				//fprintf(stderr, "errno: %d %s\n", errno, s_src);
 				break;
 			}
 		}
@@ -175,7 +175,7 @@ BOOL nt_read_thread(nt_2ch_model_tp modelp)
 		return	FALSE;
 	}
 
-	if(!nt_http_get(s_src, outp, s_dst, NULL, NULL)){
+	if(!nt_http_get(s_src, outp, s_dst, NULL, NULL, TRUE)){
 		free(outp);
 		return FALSE;
 	}
@@ -215,7 +215,7 @@ BOOL nt_read_thread(nt_2ch_model_tp modelp)
 	fclose(fp_src);
 	iconv_close(icd);
 	free(outp);
-	return TRUE;
+	return (threadp->num_res > 0);
 }
 
 BOOL nt_read_board(nt_2ch_model_tp modelp)
@@ -258,7 +258,7 @@ BOOL nt_read_board(nt_2ch_model_tp modelp)
 		return	FALSE;
 	}
 
-	if(!nt_http_get(data, outp, URL_2CH_BOARDMENU, NULL, NULL)){
+	if(!nt_http_get(data, outp, URL_2CH_BOARDMENU, NULL, NULL, FALSE)){
 		free(outp);
 		return FALSE;
 	}
@@ -287,7 +287,7 @@ BOOL nt_read_board(nt_2ch_model_tp modelp)
 		while(0	<	n_src){
 			size_t sz = iconv(icd, &p_src, &n_src, &p_dst, &n_dst);
 			if(sz == -1){
-				fprintf(stderr, "errno: %d %s\n", errno, s_src);
+				//fprintf(stderr, "errno: %d %s\n", errno, s_src);
 				break;
 			}
 		}
