@@ -161,6 +161,7 @@ nt_res_tp nt_res_alloc(nt_thread_tp threadp,
 		free(resp);
 		return NULL;
 	}
+	resp->msg_header_line_num = 0;
 	resp->msg_line_num = 0;
 	resp->msg_line_linkp = NULL;
 
@@ -362,11 +363,17 @@ void nt_res_msg_free(nt_res_tp ptr)
 		curp = ptr->msg_line_linkp->next;
 		while(curp != ptr->msg_line_linkp){
 			nextp = curp->next;
+			/* We must not free the data pointer
+			 * except the first one
+			 * because they're alocated as a same block.
+			 * This is wrong --> free(curp->data);
+			 */
 			free(curp);
 			curp = nextp;
 		}
 		free(curp->data);
 		free(curp);
+		ptr->msg_line_linkp = NULL;
 	}
 }
 
