@@ -76,12 +76,13 @@ int nt_add_wnstr(WINDOW *w, wchar_t *wc, attr_t attr, size_t colmns)
 	c.chars[1] = L'\0';
 	len = wcslen(wc);
 	for(i = 0; i < len && col < colmns; i++){
-		if(wc[i] > 127){
+		if(wc[i] <= 128 || 
+				(wc[i] >= 0xff66 && wc[i] <= 0xff9d)){
+			col++;
+		}else{
 			col += 2;
 			if(col >= colmns)
 				break;
-		}else{
-			col++;
 		}
 		c.chars[0] = wc[i];
 		if(ERR == wadd_wch(w, &c))
@@ -101,9 +102,9 @@ int nt_get_wc_count_within_colmns(wchar_t *wc, size_t colmns)
 	for(i = 0; i < len; i++){
 		ch = wc[i];
 		if(ch <= 128 || (ch >= 0xff66 && ch <= 0xff9d))
-			col += 2;
-		else
 			col++;
+		else
+			col += 2;
 		if(col > colmns)
 			return i;
 	}
