@@ -3,8 +3,10 @@
 #include <assert.h>
 #include <wchar.h>
 #include <ncursesw/ncurses.h>
+#include <regex.h>
 
 #include	"env.h"
+#include	"error.h"
 #include	"utils/nt_std_t.h"
 #include	"_2ch/_2ch.h"
 #include	"_2ch/model_2ch.h"
@@ -18,9 +20,17 @@ typedef struct tag_ctx_threadlist_t
 	int cursor_pos;
 	int scroll_pos;
 
+	regex_t regex;
+	BOOL regex_init;
+	int sel_thread_no;
+
 } ctx_threadlist_t;
 
 
+static BOOL search_line_asc(regex_t *regexp, 
+				nt_link_tp threadlistp, int *sel_thread_no);
+static BOOL search_line_desc(regex_t *regexp, 
+				nt_link_tp threadlistp, int *sel_thread_no);
 static ctx_threadlist_tp init_context(nt_2ch_model_tp modelp);
 
 int disp_threadlist(nt_window_tp wp, nt_2ch_model_tp modelp)
@@ -33,6 +43,7 @@ int disp_threadlist(nt_window_tp wp, nt_2ch_model_tp modelp)
 	int btm_cur, num, wlines;
 	wchar_t buf[16];
 	attr_t attr;
+	BOOL search_asc;
 
 	ctxp = (ctx_threadlist_tp)wp->data;
 	if(!ctxp){
@@ -46,6 +57,8 @@ int disp_threadlist(nt_window_tp wp, nt_2ch_model_tp modelp)
 	boardp = modelp->selected_boardp;
 
 	switch(ch){
+	case NT_KEY_REFRESH: 
+		return DISP_STATE_REFRESH;
 	case NT_KEY_CLOSE:
 		return DISP_STATE_BOARDMENU;
 	case NT_KEY_BOTTOM:
@@ -60,12 +73,46 @@ int disp_threadlist(nt_window_tp wp, nt_2ch_model_tp modelp)
 		ctxp->scroll_pos = wlines;
 		//ctxp->scroll_pos += wlines%2;
 		break;
+	case NT_KEY_COMMAND2:
+	case NT_KEY_COMMAND3:
+		search_asc = (ch == NT_KEY_COMMAND2);
+		if(wp->cmd_param && wp->cmd_param[0] != '\0'){
+			if(0 != regcomp(&(ctxp->regex),
+					wp->cmd_param, REG_EXTENDED)){
+				if(ctxp->regex_init){
+					regfree(&(ctxp->regex));
+					ctxp->regex_init = FALSE;
+					break;
+				}
+			}
+			if(!ctxp->regex_init)
+			    ctxp->regex_init = TRUE;
+		}
+		if(!ctxp->regex_init)
+		    break;
+		if(search_asc){
+		    if(!search_line_asc(&(ctxp->regex), boardp->threadlistp,
+			        &ctxp->sel_thread_no)){
+				wp->status_msg = NT_ERR_MSG_SEARCH_NOT_FOUND;
+				break;
+			}
+		}else{
+		    if(!search_line_desc(&(ctxp->regex), boardp->threadlistp,
+			        &ctxp->sel_thread_no)){
+				wp->status_msg = NT_ERR_MSG_SEARCH_NOT_FOUND;
+				break;
+			}
+		}
+		num = ctxp->sel_thread_no;
+		//fall through
 	case NT_KEY_COMMAND1:
-		assert(wp->cmd_param);
-		num = atoi(wp->cmd_param);
-		if(0 == num)
-			break;
-		num--;
+		if(ch == NT_KEY_COMMAND1){
+			assert(wp->cmd_param);
+			num = atoi(wp->cmd_param);
+			if(0 == num)
+				break;
+			num--;
+		}
 		wlines = ctxp->thread_num - num;
 		if(wlines <= 0)
 			break;
@@ -198,7 +245,7 @@ static ctx_threadlist_tp init_context(nt_2ch_model_tp modelp)
 {
 	ctx_threadlist_tp ctxp;
 	nt_board_tp boardp;
-	ctxp = (ctx_threadlist_tp)calloc(1,sizeof(nt_2ch_model_t));
+	ctxp = (ctx_threadlist_tp)calloc(1,sizeof(ctx_threadlist_t));
 
 	if(!ctxp)
 		return NULL;
@@ -206,6 +253,9 @@ static ctx_threadlist_tp init_context(nt_2ch_model_tp modelp)
 	boardp = modelp->selected_boardp;
 	ctxp->thread_num = nt_link_num(boardp->threadlistp);
 
+	ctxp->regex_init = FALSE;
+	ctxp->sel_thread_no = -1;
+
 	return ctxp;
 }
 
@@ -219,4 +269,103 @@ void free_threadlist_ctx(void *ptr)
 }
 
 
+static BOOL search_line_asc(regex_t *regexp, 
+				nt_link_tp threadlistp, int *sel_thread_no)
+{
+	wchar_t *cptr;
+	nt_thread_tp threadp;
+	nt_link_tp clistp;
+	size_t nmatch = 5;
+	regmatch_t pmatch[5];
+	char buf[256];
+	int cur;
+
+	clistp = threadlistp;
+	cur = 0;
+	do{
+		if(cur > *sel_thread_no){
+			threadp = (nt_thread_tp)clistp->data;
+			cptr = threadp->name;
+			if(0 < wcstombs(buf, cptr, sizeof(buf))){
+			    if(0 == regexec(regexp, buf,
+					    nmatch, pmatch, 0)){
+					*sel_thread_no = cur;
+					return TRUE;
+				}
+			}
+		}
+		cur++;
+		clistp = clistp->next;
+	}while(clistp != threadlistp);
+	clistp = threadlistp;
+	cur = 0;
+	do{
+		if(cur > *sel_thread_no)
+			break;
+
+		threadp = (nt_thread_tp)clistp->data;
+		cptr = threadp->name;
+		if(0 < wcstombs(buf, cptr, sizeof(buf))){
+		    if(0 == regexec(regexp, buf,
+				    nmatch, pmatch, 0)){
+				*sel_thread_no = cur;
+				return TRUE;
+			}
+		}
+		cur++;
+		clistp = clistp->next;
+	}while(clistp != threadlistp);
+	return FALSE;
+}
+static BOOL search_line_desc(regex_t *regexp, 
+				nt_link_tp threadlistp, int *sel_thread_no)
+{
+	wchar_t *cptr;
+	nt_thread_tp threadp;
+	nt_link_tp clistp;
+	size_t nmatch = 5;
+	regmatch_t pmatch[5];
+	char buf[256];
+	int cur;
+	int num;
+
+	clistp = threadlistp->prev;
+	num = nt_link_num(threadlistp);
+	cur = num - 1;
+	do{
+		if(cur < *sel_thread_no){
+			threadp = (nt_thread_tp)clistp->data;
+			cptr = threadp->name;
+			if(0 < wcstombs(buf, cptr, sizeof(buf))){
+			    if(0 == regexec(regexp, buf,
+					    nmatch, pmatch, 0)){
+					*sel_thread_no = cur;
+					return TRUE;
+				}
+			}
+		}
+		cur--;
+		clistp = clistp->prev;
+	}while(clistp != threadlistp->prev);
+	clistp = threadlistp->prev;
+	cur = num;
+	do{
+		if(cur < *sel_thread_no)
+			break;
+
+		threadp = (nt_thread_tp)clistp->data;
+		cptr = threadp->name;
+		if(0 < wcstombs(buf, cptr, sizeof(buf))){
+		    if(0 == regexec(regexp, buf,
+				    nmatch, pmatch, 0)){
+				*sel_thread_no = cur;
+				return TRUE;
+			}
+		}
+		cur--;
+		clistp = clistp->prev;
+	}while(clistp != threadlistp->prev);
+	return FALSE;
+}
+
 
