@@ -277,19 +277,9 @@ void nt_stack_free(nt_stack_tp ptr, nt_memfree_tp free_func)
 	}
 	if(ptr->linkp){
 		nt_all_link_free(ptr->linkp, free_func);
-		/*linkp = ptr->linkp;
-		do{
-			(free_func)(linkp->data);
-			linkp = linkp->next;
-		}while(linkp != ptr->linkp);*/
 	}
 	if(ptr->unlinkedp){
 		nt_all_link_free(ptr->unlinkedp, free_func);
-		/*linkp = ptr->unlinkedp;
-		do{
-			(free_func)(linkp->data);
-			linkp = linkp->next;
-		}while(linkp != ptr->unlinkedp);*/
 	}
 	free(ptr);	
 }
